from unittest import TestCase

from ..code import error_dict, success_dict


class CodeTest(TestCase):
    def test_response(self):
        assert "code" in error_dict("test")
        assert "code" in success_dict("test", {})



# coding: utf-8

from enum import IntEnum
from typing import Any, Dict

from django.http import JsonResponse


class Code(IntEnum):
    ERROR = 0
    SUCCESS = 1
    FAILED = 2
    ACCESS_EXPIRED = 3
    REFRESH_EXPIRED = 4


def error_dict(msg: str) -> dict:
    return {"code": Code.FAILED.value, "msg": msg, "data": {}}


def error_response(msg: str) -> JsonResponse:
    return JsonResponse(data=error_dict(msg))


def success_dict(msg: str, data: dict) -> dict:
    return {"code": Code.SUCCESS, "msg": msg, "data": data}


def success_response(msg: str, data: dict) -> JsonResponse:
    return JsonResponse(data=success_dict(msg, data))


def error_data(data: Any) -> Dict:
    return {"code": Code.FAILED, "msg": "error", "data": data}


class ResponseMixin:
    def success_dict(self, msg, data):
        return success_dict(msg, data)

    def error_dict(self, msg):
        return error_dict(msg)



from .case import ThemeBaseCase
from .theme import ThemeCase


class EnglishThemeTest(ThemeBaseCase):
    def setUp(self) -> None:
        self.theme_case = ThemeCase()

    def test_english_name(self):
        """TODO: not finished yet. """



# Create your views here.
from typing import TYPE_CHECKING
from rest_framework.response import Response
from rest_framework.views import APIView

from django_theme.libs.code import error_dict, success_dict

from .models import Theme


class SystemStyleView(APIView):
    def get(self, request):
        style = Theme.objects.first()
        if not style:
            return Response(error_dict(msg="Theme config has not be set."))
        theme_dict = style.to_dict()
        return Response(success_dict(msg="success", data=theme_dict))



from django_theme.tests.case import ThemeBaseCase


class ThemeTest(ThemeBaseCase):
    def test_without_theme(self):
        response = self.client.get(self.api_theme_url)
        self.assertEqual(response.status_code, 200)



from django.db import models


# Create your models here.
class Theme(models.Model):
    title = models.CharField("标题名称", max_length=64, default="Andon管理系统")
    theme_id = models.AutoField(primary_key=True)
    theme_color = models.CharField(
        "主题色", max_length=128, null=True, blank=True, default="#3dcd58"
    )
    aux_color = models.CharField(
        "辅色", max_length=128, null=True, blank=True, default="#696969"
    )
    corporate_logo = models.ImageField(
        "LOGO", max_length=300, null=True, upload_to="theme"
    )
    tab_logo = models.ImageField(
        "TAB_LOGO", max_length=300, null=True, upload_to="theme"
    )
    login_logo = models.ImageField(
        "LOGIN_LOGO", max_length=300, null=True, upload_to="theme"
    )

    def to_dict(self):
        return {
            "title": self.title,
            "corporate_logo": self.corporate_logo.url,
            "tab_logo": self.tab_logo.url,
            "login_logo": self.login_logo.url,
            "self_color": self.theme_color,
            "aux_color": self.aux_color,
        }



import logging
from typing import Any, Callable, Tuple, Type

from django.test import Client
from django.urls import resolve, reverse
from django.views import View

from django_theme.libs.code import Code

logger = logging.getLogger(__name__)


def assert_instance(data: Any, types: Type):
    assert isinstance(data, types), f"type is {type(data)}"


def assert_key(key: Any, data: dict):
    if key not in data.keys():
        raise AssertionError(f"{key} not in {data.keys()}")


def assert_attr(objects: Any, keys: Tuple[str, ...]):
    for key in keys:
        assert hasattr(objects, key), f"{objects} not has {key}"


def assert_export_file_type(resp):
    got = resp["Content-Type"]
    want = "application/octet-stream"
    assert_equal(got, want)


def assert_export_file_details(resp, filename: str):
    got = resp.get("Content-Disposition")
    want = f"attachment; filename={filename}"
    assert_equal(got, want)


def assert_response_get(client: Client, urls: str):
    resp = client.get(urls)
    assert resp.status_code == 200

    assert_response_format(resp.json())
    return resp


def assert_keys(keys: Tuple[str, ...], data: dict):
    for key in keys:
        if key not in data.keys():
            raise AssertionError(f"{key} not in {data.keys()}")


def assert_equal(got: Any, want: Any):
    if want != got:
        assert False, f"{got} != {want}"


def assert_view(urls: str, view_func: Callable):
    got = resolve(urls).func
    assert_equal(got, view_func)


def assert_urls(urls: str, view_name: str):
    got = reverse(view_name)
    want = urls
    assert_equal(got, want)


def assert_class_view(urls: str, view: Type[View]):
    got = resolve(urls).func
    want = view.as_view()
    assert_equal(got.view_class, want.view_class)


def assert_response(client: Client, urls: str, data: dict = None):
    resp = client.post(urls, data=data)
    assert resp.status_code == 200

    assert_response_format(resp.json())
    return resp


def assert_key_value_types(
        data: dict, keys: Tuple[str, ...], value_types: Tuple[Any, ...]
):
    for key, value_type in zip(keys, value_types):
        assert key in data.keys(), (key, data)
        assert isinstance(data[key], value_type), (key, value_type)


def assert_response_format(resp_json: dict):
    # test response structure
    keys = ("code", "msg", "data")
    assert_keys(keys, resp_json)


def assert_code(resp_json: dict, code: Code):
    if Code(resp_json.get("code")) != code:
        raise AssertionError(f"code: {code} != code in {resp_json}")



import pytest
from django.test import Client

from .. import test
from ..code import Code


def test_assert_instance():
    test_data = "test"
    test.assert_instance(test_data, str)
    with pytest.raises(AssertionError):
        test.assert_instance(test_data, int)


def test_dict_has_key():
    key = "purchase_category"
    got = {"purchase_category": "test"}
    test.assert_key(key, got)
    key1 = "purchase"
    with pytest.raises(AssertionError):
        test.assert_key(key1, got)


def test_object_has_attr():
    keys = ("SUCCESS", "FAILED")
    test.assert_attr(Code, keys)
    case2 = ("FALSE", "test")
    with pytest.raises(AssertionError):
        test.assert_attr(Code, case2)


def test_assert_response_get():
    with pytest.raises(AssertionError):
        test.assert_response_get(Client(), "/test")


def test_export_file_type_and_details():
    data = {"test": "test"}
    resp = Client().get("/test", data)
    with pytest.raises(AssertionError):
        test.assert_export_file_type(resp)
        test.assert_export_file_details(resp, "test")


def test_assert_keys():
    data = {"code": 1, "msg": "success", "data": "hello"}

    keys = ("code", "msg", "data")
    test.assert_keys(keys, data)

    keys = ("test", "msg")
    with pytest.raises(AssertionError):
        test.assert_keys(keys, data)


def test_assert_equal():
    a, b = 1, 2
    with pytest.raises(AssertionError):
        test.assert_equal(a, b)

    a, b = 1, 1
    test.assert_equal(a, b)


def test_assert_structure():
    with pytest.raises(AssertionError):
        test.assert_response(Client(), "/test")

    with pytest.raises(AssertionError):
        test.assert_response(Client(), "/test", data={})


def test_utils_assert_in():
    data = {"code": 0, "msg": 1, "data": {"hello": "world"}}
    test.assert_response_format(data)

    with pytest.raises(AssertionError):
        data.pop("code")
        test.assert_response_format(data)


def test_utils_success():
    data = {"code": -1, "msg": 1, "data": {"hello": "world"}}

    with pytest.raises(ValueError):
        test.assert_code(data, Code.SUCCESS)

    data = {"code": 1, "msg": 1, "data": {"hello": "world"}}
    test.assert_code(data, Code.SUCCESS)


def test_assert_key_value():
    data = {"page": 1, "count": 2, "groups": []}

    keys = ("page", "count", "groups")
    value_types = (int, int, list)
    test.assert_key_value_types(data, keys, value_types)

    with pytest.raises(AssertionError):
        value_types = value_types[1:]
        test.assert_key_value_types(data, keys, value_types)



# Generated by Django 3.0 on 2021-09-27 06:46

from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Theme',
            fields=[
                ('title', models.CharField(default='Andon管理系统', max_length=64, verbose_name='标题名称')),
                ('theme_id', models.AutoField(primary_key=True, serialize=False)),
                ('theme_color',
                 models.CharField(blank=True, default='#3dcd58', max_length=128, null=True, verbose_name='主题色')),
                ('aux_color',
                 models.CharField(blank=True, default='#696969', max_length=128, null=True, verbose_name='辅色')),
                (
                'corporate_logo', models.ImageField(max_length=300, null=True, upload_to='theme', verbose_name='LOGO')),
                ('tab_logo', models.ImageField(max_length=300, null=True, upload_to='theme', verbose_name='TAB_LOGO')),
            ],
        ),
    ]



# Generated by Django 3.2.7 on 2021-10-04 13:02

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('django_theme', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='theme',
            name='login_logo',
            field=models.ImageField(max_length=300, null=True, upload_to='theme', verbose_name='LOGIN_LOGO'),
        ),
    ]



from django.core.files import File

from ..models import Theme


class ThemeCase:
    """
    create a sample theme
    """

    def __init__(self, corporate_logo_path="theme/theme/logo.jpg",
                 tab_logo_path="theme/theme/tab_logo.png",
                 login_logo_path="theme/theme/login_logo.jpg"):
        self.theme = {
            "title": "天和双利",
            "theme_color": "#e60000",
            "aux_color": "#666666",
        }
        self.theme_object = self.create_theme()

        self.theme_object.corporate_logo.save(
            "logo.jpg", File(open(corporate_logo_path, "rb"))
        )
        self.theme_object.tab_logo.save(
            "tab_logo.png", File(open(tab_logo_path, "rb"))
        )
        self.theme_object.login_logo.save(
            "login_logo.jpg", File(open(login_logo_path, "rb"))
        )

    def attach_file(self):
        return [
            (
                "corporate_logo",
                ("logo.jpg", open("theme/theme/logo.jpg", "rb"), "image/jpg"),
            ),
            (
                "tab_logo",
                (
                    "tab_logo.png",
                    open("theme/theme/tab_logo.png", "rb"),
                    "image/png"
                ),
            ),
            (
                "login_logo",
                (
                    "login_logo.jpg",
                    open("theme/theme/login_logo.jpg", "rb"),
                    "image/jpg"
                ),
            ),
        ]

    def create_theme(self):
        return Theme.objects.create(**self.theme)



from django.contrib import admin

from .models import Theme


# Register your models here.


@admin.register(Theme)
class ThemeAdmin(admin.ModelAdmin):
    list_display = ("title", "theme_color", "aux_color")



from django_theme.tests.case import ThemeBaseCase
from django_theme.libs.code import Code

from .theme import ThemeCase


class ThemeFunctionalTest(ThemeBaseCase):
    def setUp(self) -> None:
        ThemeCase()
        super().setUp()

    def assertResponse(self) -> dict:
        response = self.client.get(path=self.api_theme_url)
        self.assertEqual(response.status_code, 200)

        json_data = response.json()
        self.assertEqual(json_data.get("code"), Code.SUCCESS, msg=json_data.get("msg"))
        return json_data

    def test_get_theme_details(self):
        """
        获取系统配置：主题色，logo等：每个系统只会有一个配置所以获取第一个可以满足需求
        不鉴权
        """

        theme_data = self.assertResponse().get("data").get("title")
        self.assertEqual(theme_data, "天和双利")

    def test_get_theme_logo(self):
        json_data = self.assertResponse()
        tab_logo_path = json_data.get("data").get("tab_logo")
        self.assertTrue(
            tab_logo_path.startswith("/theme/theme"), msg=tab_logo_path
        )

    def test_get_login_logo(self):
        json_data = self.assertResponse()
        login_logo_path = json_data.get("data").get("login_logo")
        self.assertTrue(
            login_logo_path.startswith("/theme/theme"), msg=login_logo_path
        )
